(in-package :pong.client)

(defclass game-scene (scene)
  ((game :initarg :game :initform nil :reader game-scene-game)
   (left-score :initarg :left-score :initform nil :reader game-scene-left-score)
   (right-score :initarg :right-score :initform nil :reader game-scene-right-score)
   (left-paddle :initarg :left-paddle :initform nil :reader game-scene-left-paddle)
   (right-paddle :initarg :right-paddle :initform nil :reader game-scene-right-paddle)
   (left-player :initarg :left-player :initform nil :reader game-scene-left-player)
   (right-player :initarg :right-player :initform nil :reader game-scene-right-player)
   (ball :initarg :ball :initform nil :reader game-scene-ball)
   (line :initarg :line :initform nil :reader game-scene-line)
   (quit-menu :initarg :quit-menu :initform nil :reader game-scene-quit-menu)
   (quit-menu-text :initarg :quit-menu-text :initform nil :reader game-scene-quit-menu-text)
   (quit-menu-yes :initarg :quit-menu-yes :initform nil :reader game-scene-quit-menu-yes)
   (quit-menu-no :initarg :quit-menu-no :initform nil :reader game-scene-quit-menu-no)
   (code :initarg :code :initform nil :reader game-scene-code)
   (popup :initarg :popup :initform nil :reader game-scene-popup)
   (popup-text :initarg :popup-text :initform nil :reader game-scene-popup-text)))

(defun open-game (game)
  (let ((root-element (gui:make-rectangle :color r:+darkgray+
					  :h-align :center
					  :v-align :middle))
	(left-score (gui:make-text        :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
				          :h-align :right
					  :y *score-padding*))
	(right-score (gui:make-text       :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :y *score-padding*))
	(left-paddle (gui:make-rectangle  :color r:+white+
					  :v-align :middle))
	(right-paddle (gui:make-rectangle :color r:+white+
					  :h-align :right
					  :v-align :middle))
	(left-player (gui:make-text       :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :x *score-padding*
					  :y *score-padding*))
	(right-player (gui:make-text      :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :h-align :right
					  :y *score-padding*))
	(ball (gui:make-rectangle         :color r:+white+
					  :h-align :center
					  :v-align :middle))
	(line (gui:make-line              :thickness 4
					  :color r:+white+))
	(quit-menu (gui:make-rectangle    :color r:+black+
					  :h-align :center
					  :v-align :middle
					  :visible nil))
	(quit-menu-text (gui:make-text    :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :y *score-padding*
					  :h-align :center
					  :text "Do you really want to quit the game?"))
	(quit-menu-yes (gui:make-text     :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :h-align :center
					  :text "Yes"))
	(quit-menu-no (gui:make-text      :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :h-align :center
					  :text "No"))
	(code (gui:make-text              :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :x *score-padding*
					  :v-align :bottom))
	(popup (gui:make-rectangle        :color r:+black+
					  :h-align :center
					  :v-align :middle
					  :visible nil))
	(popup-text (gui:make-text        :color r:+white+
					  :font *score-font*
					  :font-size *score-txt-size*
					  :y *score-padding*
					  :x *score-padding*
					  :text "Waiting for opponent...")))

    (gui:add-children quit-menu
		      quit-menu-text
		      quit-menu-yes
		      quit-menu-no)

    (gui:add-children popup popup-text)

    (gui:add-children root-element
		      quit-menu
		      popup
		      left-score
		      right-score
		      left-paddle
		      right-paddle
		      ball
		      line
		      left-player
		      right-player
		      code)

    (g:on-init game)

    (make-instance 'game-scene :game game
			       :root-element root-element
			       :left-score left-score
			       :right-score right-score
			       :left-paddle left-paddle
			       :right-paddle right-paddle
			       :left-player left-player
			       :right-player right-player
			       :ball ball
			       :line line
			       :quit-menu quit-menu
			       :quit-menu-text quit-menu-text
			       :quit-menu-yes quit-menu-yes
			       :quit-menu-no quit-menu-no
			       :code code
			       :popup popup
			       :popup-text popup-text)))

(defun position-score (el direction)
  (setf (gui:x el) (funcall direction (floor (gui:w (gui:parent el)) 2) *score-padding*)))

(defun position-paddle (paddle el)
  (let ((paddle-y (g:paddle-y paddle))
	(parent (gui:parent el)))
    (setf (gui:w el) (* g:+paddle-width+ (gui:w parent)))
    (setf (gui:h el) (* g:+paddle-height+ (gui:h parent)))
    (setf (gui:y el) (* paddle-y (gui:h parent)))))

(defun position-right-paddle (paddle el)
  (position-paddle paddle el)
  (setf (gui:x el) (gui:w (gui:parent el))))

(defun position-ball (ball el)
  (let ((ball-x (g:ball-x ball))
	(ball-y (g:ball-y ball))
	(parent (gui:parent el)))
    (setf (gui:w el) (* g:+ball-radius+ (gui:w parent) 2))
    (setf (gui:h el) (* g:+ball-radius+ (gui:h parent) 2))
    (setf (gui:x el) (* ball-x (gui:w parent)))
    (setf (gui:y el) (* ball-y (gui:h parent)))))

(defun position-line (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2))
  (setf (gui:end-x el) (floor (gui:w (gui:parent el)) 2))
  (setf (gui:end-y el) (gui:h (gui:parent el))))

(defun position-right-player (el)
  (setf (gui:x el) (- (gui:w (gui:parent el)) *score-padding*)))

(defun position-quit-menu (menu text yes no)
  (setf (gui:x menu) (floor (gui:w (gui:parent menu)) 2))
  (setf (gui:y menu) (floor (gui:h (gui:parent menu)) 2))
  (setf (gui:w menu) (+ (gui:w text) (* *score-padding* 2)))
  (setf (gui:x text) (floor (gui:w menu) 2))
  (setf (gui:x yes) (floor (gui:w menu) 3))
  (setf (gui:x no) (floor (gui:w menu) 3/2))
  (setf (gui:y yes) (+ (gui:y text) (gui:h text) *score-padding*))
  (setf (gui:y no) (+ (gui:y text) (gui:h text) *score-padding*))
  (setf (gui:h menu) (+ (gui:y yes) (gui:h yes) *score-padding*)))

(defun position-popup (popup text)
  (setf (gui:x popup) (floor (gui:w (gui:parent popup)) 2))
  (setf (gui:y popup) (floor (gui:h (gui:parent popup)) 2))
  (setf (gui:w popup) (+ (gui:w text) (* *score-padding* 2)))
  (setf (gui:h popup) (+ (gui:h text) (* *score-padding* 2))))

(defun position-code (el)
  (setf (gui:y el) (- (gui:h (gui:parent el)) *score-padding*)))

(defmethod on-update ((scene game-scene) timelapse)
  (g:on-update (game-scene-game scene) timelapse)

  (update-text-if-hovered (game-scene-quit-menu-yes scene) "Yes")
  (update-text-if-hovered (game-scene-quit-menu-no scene) "No")
  (when (gui:clickedp (game-scene-quit-menu-no scene) r:+mouse-button-left+)
    (g:resume (game-scene-game scene))
    (setf (gui:visible (game-scene-quit-menu scene)) nil))
  (when (r:is-key-pressed r:+key-escape+)
    (g:pause (game-scene-game scene))
    (setf (gui:visible (game-scene-quit-menu scene)) t))
  (when (gui:clickedp (game-scene-quit-menu-yes scene) r:+mouse-button-left+)
    (g:on-quit (game-scene-game scene))
    (open-main-menu)))

(defmethod on-draw ((scene game-scene))
  (with-slots (game root-element left-score right-score left-paddle right-paddle ball line left-player right-player quit-menu quit-menu-text quit-menu-yes quit-menu-no code popup popup-text)
      scene
    (let ((game-state (g:game-state game)))
      (position-root root-element)
      (setf (gui:text left-score) (format nil "~a" (g:state-left-score game-state)))
      (setf (gui:text right-score) (format nil "~a" (g:state-right-score game-state)))
      (position-score left-score #'-)
      (position-score right-score #'+)
      (position-paddle (g:state-left-paddle game-state) left-paddle)
      (position-right-paddle (g:state-right-paddle game-state) right-paddle)
      (position-ball (g:state-ball game-state) ball)
      (position-line line)
      (setf (gui:text left-player) (g:state-left-player game-state))
      (setf (gui:text right-player) (g:state-right-player game-state))
      (position-right-player right-player)
      (position-quit-menu quit-menu quit-menu-text quit-menu-yes quit-menu-no)
      (setf (gui:text code) (when (eq (type-of game) 'online-game)
			      (format nil "Code: ~a" (g:game-code game))))
      (position-code code)
      (position-popup popup popup-text))))
