(in-package #:gui)

(defclass element ()
  ((x        :initarg :x        :initform 0.0   :accessor x)
   (y        :initarg :y        :initform 0.0   :accessor y)
   (w        :initarg :w        :initform 0.0   :accessor w)
   (h        :initarg :h        :initform 0.0   :accessor h)
   (h-align  :initarg :h-align  :initform :left :accessor h-align)
   (v-align  :initarg :v-align  :initform :top  :accessor v-align)
   (offset-x :initarg :offset-x :initform 0.0   :reader offset-x)
   (offset-y :initarg :offset-y :initform 0.0   :reader offset-y)
   (screen-x :initarg :screen-x :initform 0.0   :reader screen-x)
   (screen-y :initarg :screen-y :initform 0.0   :reader screen-y)
   (visible  :initarg :visible  :initform t     :accessor visible)
   (parent   :initarg :parent   :initform nil   :reader parent)
   (children :initarg :children :initform nil   :reader children)))

(defmethod hoveredp ((el element))
  (let ((mpos (r:get-mouse-position)))
    (with-slots (screen-x screen-y w h) el
      (and (< (v:vx mpos) (+ screen-x w))
	   (>= (v:vx mpos) screen-x)
	   (< (v:vy mpos) (+ screen-y h))
	   (>= (v:vy mpos) screen-y)))))

(defmethod clickedp ((el element) button)
  (and (r:is-mouse-button-pressed button)
       (hoveredp el)))

(defmethod set-screen-x ((el element) x)
  (setf (slot-value el 'screen-x) x)
  (mapc (lambda (el) (set-screen-x el (+ x (offset-x el))))
	(children el)))

(defun update-x (el)
  (with-slots (x w h-align offset-x screen-x) el
    (incf screen-x (- offset-x))
    (setf offset-x (case h-align
		     (:center (- x (floor w 2)))
		     (:right (- x w))
		     (otherwise x)))
    (set-screen-x el (+ screen-x offset-x))))

(defmethod (setf x) (value (el element))
  (setf (slot-value el 'x) value)
  (update-x el))

(defmethod (setf w) (value (el element))
  (setf (slot-value el 'w) value)
  (unless (eq (slot-value el 'h-align) :left)
    (update-x el)))

(defmethod (setf h-align) (value (el element))
  (setf (slot-value el 'h-align) value)
  (update-x el))

(defmethod set-screen-y ((el element) y)
  (setf (slot-value el 'screen-y) y)
  (mapc (lambda (el) (set-screen-y el (+ y (offset-y el))))
	(children el)))

(defun update-y (el)
  (with-slots (y h v-align offset-y screen-y) el
    (incf screen-y (- offset-y))
    (setf offset-y (case v-align
		     (:middle (- y (floor h 2)))
		     (:bottom (- y h))
		     (otherwise y)))
    (set-screen-y el (+ screen-y offset-y))))

(defmethod (setf y) (value (el element))
  (setf (slot-value el 'y) value)
  (update-y el))

(defmethod (setf h) (value (el element))
  (setf (slot-value el 'h) value)
  (unless (eq (slot-value el 'v-align) :top)
    (update-y el)))

(defmethod (setf v-align) (value (el element))
  (setf (slot-value el 'v-align) value)
  (update-y el))

(defmethod remove-child ((parent element) (child element))
  (setf (slot-value parent 'children)
	(remove child (children parent))))

(defmethod remove-children ((parent element) &rest children)
  (mapc #'(lambda (child) (remove-child parent child)) children))

(defmethod add-child ((parent element) (child element))
  (when (parent child)
    (remove-child (parent child) child))
  (push child (slot-value parent 'children))
  (setf (slot-value child 'parent) parent))

(defmethod add-children ((parent element) &rest children)
  (mapc #'(lambda (child) (add-child parent child)) children))

(defmethod calculate-size ((el element)))

(defmethod update ((el element))
  (calculate-size el)
  (update-y el)
  (update-x el)
  el)

(defmethod draw ((el element)))
(defmethod draw :after ((el element))
  (when (visible el) 
    (mapc #'draw (children el))))

(defmacro make-element (&rest args)
  `(update (make-instance 'element ,@args)))
  
