(in-package #:gui)

(defclass text (element)
  ((w         :initarg :w         :initform 0.0       :reader w)
   (h         :initarg :h         :initform 0.0       :reader h)
   (text      :initarg :text      :initform ""        :accessor text)
   (color     :initarg :color     :initform r:+black+ :accessor color)
   (font      :initarg :font      :initform nil       :accessor font)
   (font-size :initarg :font-size :initform 32        :accessor font-size)
   (spacing   :initarg :spacing   :initform 1.0       :accessor spacing)))

(defun load-font (font font-size)
  (let ((font-family (or (gethash font *fonts*)
			 (setf (gethash font *fonts*) (make-hash-table)))))
    (or (gethash font-size font-family)
	(setf (gethash font-size font-family)
	      (r:load-font-ex (format nil "~a~a" (uiop/os:getcwd) font) font-size (cffi:null-pointer) 0)))))

(defmethod draw ((text text))
  (with-slots (font font-size text screen-x screen-y color spacing visible) text
    (when (and visible text)
      (r:draw-text-ex (load-font font font-size)
		      text
		      (v:vec (float screen-x) (float screen-y))
		      (float font-size)
		      spacing
		      color))))

(defmethod calculate-size ((txt text))
  (with-slots (w h font font-size text spacing) txt
    (when (and font text)
      (let ((size (r:measure-text-ex (load-font font font-size)
				     text
				     (float font-size)
				     spacing)))
	(setf w (v:vx size))
	(setf h (v:vy size))
	(update-x txt)
	(update-y txt)))))

(defmethod (setf text) (value (txt text))
  (setf (slot-value txt 'text) value)
  (update txt))

(defmethod (setf font) (value (txt text))
  (setf (slot-value txt 'font) value)
  (update txt))

(defmethod (setf font) (value (txt text))
  (setf (slot-value txt 'font) value)
  (update txt))

(defmethod (setf font-size) (value (txt text))
  (setf (slot-value txt 'font-size) value)
  (update txt))

(defmethod (setf spacing) (value (txt text))
  (setf (slot-value txt 'spacing) value)
  (update txt))

(defmacro make-text (&rest args)
  `(update (make-instance 'text ,@args)))

