(in-package :pong.client)

(defclass join-online-game-scene (scene)
  ((name :initform "" :accessor join-online-game-scene-name)
   (code :initform "" :accessor join-online-game-scene-code)
   (title :initarg :title :initform nil :reader join-online-game-scene-title)
   (name-field  :initarg :name-field
	        :initform nil
	        :reader join-online-game-scene-name-field)
   (name-prompt :initarg :name-prompt
		:initform nil
		:reader join-online-game-scene-name-prompt)
   (code-field  :initarg :code-field
	        :initform nil
	        :reader join-online-game-scene-code-field)
   (code-prompt :initarg :code-prompt
		:initform nil
		:reader join-online-game-scene-code-prompt)
   (join-button :initarg :join-button
		:initform nil
		:reader join-online-game-scene-join-button)
   (items-group :initarg :items-group
		:initform nil
		:reader join-online-game-scene-items-group)
   (popup :initarg :popup :initform nil :reader join-online-game-scene-popup)
   (popup-text :initarg :popup-text :initform nil :reader join-online-game-scene-popup-text)))

(defun open-join-online-game-scene ()
  (let* ((root-element (gui:make-rectangle :color r:+darkgray+
					   :h-align :center
					   :v-align :middle))
	 (title (gui:make-text :color r:+white+
			       :font *menu-font*
			       :font-size *menu-title-font-size*
			       :y *title-top-padding*
			       :h-align :center
			       :text "Join online game"))
	 (name-field (gui:make-text-field :y 0
					  :color r:+white+
					  :font *menu-font*
					  :font-size *menu-font-size*
					  :focusp t
					  :border :solid
					  :border-color r:+white+
					  :border-thickness 1
					  :cursor 0
					  :v-align :middle
					  :padding 10
					  :min-width 150))
	 (name-prompt (gui:make-text :y 0
				     :color r:+white+
				     :font *menu-font*
				     :font-size *menu-font-size*
				     :h-align :right
				     :v-align :middle
				     :text "What is your name?:"))
	 (code-field (gui:make-text-field :y (+ (gui:h name-field) *menu-padding*)
					  :color r:+white+
					  :font *menu-font*
					  :font-size *menu-font-size*
					  :focusp nil
					  :border :solid
					  :border-color r:+white+
					  :border-thickness 1
					  :cursor 0
					  :v-align :middle
					  :padding 10
					  :min-width 150))
	 (code-prompt (gui:make-text :y (+ (gui:h name-field) *menu-padding*)
				     :color r:+white+
				     :font *menu-font*
				     :font-size *menu-font-size*
				     :h-align :right
				     :v-align :middle
				     :text "What is the game code?:"))
	 (join-button-text (gui:make-text :x (* *menu-padding* 2)
					  :y (* *menu-padding* 2)
					  :color r:+darkgray+
					  :font *menu-font*
					  :font-size *menu-font-size*
					  :text "Join game"))
	 (join-button (gui:make-rectangle :y (+ (gui:y code-field)
						(gui:h code-field)
						*menu-padding*)
					  :w (+ (gui:w join-button-text) (* *menu-padding* 4))
					  :h (+ (gui:h join-button-text) (* *menu-padding* 4))
					  :h-align :middle
					  :color r:+white+))
	 (items-group (gui:make-element :h (+ (gui:y join-button) (gui:h join-button))
					:h-align :center
					:v-align :middle))
	 (popup (gui:make-rectangle       :color r:+black+
					  :h-align :center
					  :v-align :middle
					  :visible nil))
	 (popup-text (gui:make-text        :color r:+white+
					   :font *score-font*
					   :font-size *score-txt-size*
					   :y *score-padding*
					   :x *score-padding*)))

    (gui:add-children popup popup-text)
    (gui:add-children join-button join-button-text)
    (gui:add-children items-group name-field name-prompt code-field code-prompt join-button)
    (gui:add-children root-element title popup items-group)
    
    (set-scene (make-instance 'join-online-game-scene :root-element root-element
						      :title title
						      :name-field name-field
						      :name-prompt name-prompt
						      :code-field code-field
						      :code-prompt code-prompt
						      :join-button join-button
						      :items-group items-group
						      :popup popup
						      :popup-text popup-text))))

(defmethod on-update ((scene join-online-game-scene) timelapse)
  (with-slots (name code move-back name-field code-field join-button) scene
    ;; Might be interesting to add repeating keys, but I don't care.
    (when (and (gui:clickedp name-field r:+mouse-button-left+)
	       (not (gui:focusp name-field)))
      (setf (gui:focusp name-field) t)
      (setf (gui:focusp code-field) nil))

    (when (and (gui:clickedp code-field r:+mouse-button-left+)
	       (not (gui:focusp code-field)))
      (setf (gui:focusp code-field) t)
      (setf (gui:focusp name-field) nil))

    (loop for key = (r:get-key-pressed)
	  while (plusp key)
	  when (= key r:+key-escape+)
	    do (open-main-menu)
	  when (= key r:+key-backspace+)
	    do (cond ((gui:focusp name-field)
		      (setf (gui:cursor name-field)
			    (max 0 (1- (gui:cursor name-field))))
		      (setf name (str:substring 0 -1 name)))
		     ((gui:focusp code-field)
		      (setf (gui:cursor code-field)
			    (max 0 (1- (gui:cursor code-field))))
		      (setf code (str:substring 0 -1 code))))
	  when (and (>= key 32) (<= key 96))
	    do (cond ((gui:focusp name-field)
		      (incf (gui:cursor name-field))
		      (setf name (str:concat name (string (code-char key)))))
		     ((gui:focusp code-field)
		      (incf (gui:cursor code-field))
		      (setf code (str:concat code (string (code-char key)))))))

    (when (gui:clickedp join-button r:+mouse-button-left+)
      (case (join-online-game name code)
	(:game-full
	 (setf (gui:visible (join-online-game-scene-popup scene)) t)
	 (setf (gui:text (join-online-game-scene-popup-text scene)) "The game is full."))
	(:game-does-not-exist
	 (setf (gui:visible (join-online-game-scene-popup scene)) t)
	 (setf (gui:text (join-online-game-scene-popup-text scene)) "The game doesn't exist."))))))

(defmethod on-draw ((scene join-online-game-scene))
  (with-slots (name code root-element title name-field name-prompt code-field code-prompt join-button items-group popup popup-text) scene
    (position-root root-element)
    (setf (gui:x title) (floor (gui:w root-element) 2))

    (setf (gui:text name-field) name)
    (setf (gui:text code-field) code)

    (setf (gui:x items-group) (floor (gui:w root-element) 2))
    (setf (gui:y items-group) (floor (gui:h root-element) 2))
    (setf (gui:w items-group) (gui:w root-element))

    (setf (gui:x name-field) (+ (floor (gui:w root-element) 2) *score-padding*))
    (setf (gui:x name-prompt) (- (floor (gui:w root-element) 2) *score-padding*))
    (setf (gui:x code-field) (+ (floor (gui:w root-element) 2) *score-padding*))
    (setf (gui:x code-prompt) (- (floor (gui:w root-element) 2) *score-padding*))
    (setf (gui:x join-button) (floor (gui:w root-element) 2))

    (setf (gui:x popup) (floor (gui:w (gui:parent popup)) 2))
    (setf (gui:y popup) (floor (gui:h (gui:parent popup)) 2))
    (setf (gui:w popup) (+ (gui:w popup-text) (* *score-padding* 2)))
    (setf (gui:h popup) (+ (gui:h popup-text) (* *score-padding* 2)))))
