(in-package :pong.client)

(defclass local-game-1p (g:game)
  ((computer-paddle-target :initarg :computer-paddle-target
			   :initform nil
			   :accessor local-game-1p-computer-paddle-target)))

(defclass local-game-2p (g:game) ())

(defun start-1-player-game ()
  (setf *scene* (open-game (make-instance 'local-game-1p))))

(defun start-2-players-game ()
  (setf *scene* (open-game (make-instance 'local-game-2p))))

(defun handle-player-paddle (paddle timelapse upward-keys downward-keys)
  (let ((paddle-y (g:paddle-y paddle))
	(paddle-vy (g:paddle-vy paddle)))
    (setf (g:paddle-vy paddle)
	  (cond ((member-if #'r:is-key-down downward-keys) g:+paddle-speed+)
		((member-if #'r:is-key-down upward-keys) (- g:+paddle-speed+))
		(t 0.0)))
    (setf (g:paddle-y paddle)
	  (min (max (+ paddle-y (* paddle-vy timelapse)) (/ g:+paddle-height+ 2))
	       (- 1.0 (/ g:+paddle-height+ 2))))))

(defun handle-left-player (state timelapse upward-keys downward-keys)
  (handle-player-paddle (g:state-left-paddle state) timelapse upward-keys downward-keys))

(defun handle-right-player (state timelapse upward-keys downward-keys)
  (handle-player-paddle (g:state-right-paddle state) timelapse upward-keys downward-keys))

(defun handle-right-computer (state timelapse target)
  (let* ((right-paddle (g:state-right-paddle state))
	 (paddle-y (g:paddle-y right-paddle))
	 (ball (g:state-ball state))
	 (ball-x (g:ball-x ball))
	 (ball-y (g:ball-y ball))
	 (paddle-target (and target (+ paddle-y (- target (/ g:+paddle-height+ 2))))))

    
    (setf (g:paddle-vy right-paddle)
	  (if paddle-target
	      (let ((delta (/ (- ball-y paddle-target) timelapse)))
		(if (minusp delta)
		    (max delta (- g:+paddle-speed+))
		    (min delta g:+paddle-speed+)))
	      0.0))

    (setf (g:paddle-y right-paddle)
	  (min (max (+ paddle-y (* (g:paddle-vy right-paddle) timelapse))
		    (/ g:+paddle-height+ 2))
	       (- 1.0 (/ g:+paddle-height+ 2))))))

(defun point-in-rect-p (px py rx ry rw rh)
  (and (> px rx)
       (<  px (+ rx rw))
       (> py ry)
       (<  py (+ ry rh))))

(defun get-ball-paddle-collision (ball paddle paddle-x)
  (let* ((ball-x (g:ball-x ball))
	 (ball-y (g:ball-y ball))
	 (paddle-y (g:paddle-y paddle)))
    (when (or (point-in-rect-p (- ball-x g:+ball-radius+)
			       (- ball-y g:+ball-radius+)
			       paddle-x
			       (- paddle-y (/ g:+paddle-height+ 2))
			       g:+paddle-width+
			       g:+paddle-height+)
	      (point-in-rect-p (- ball-x g:+ball-radius+)
			       (+ ball-y g:+ball-radius+)
			       paddle-x
			       (- paddle-y (/ g:+paddle-height+ 2))
			       g:+paddle-width+
			       g:+paddle-height+)
	      (point-in-rect-p (+ ball-x g:+ball-radius+)
			       (- ball-y g:+ball-radius+)
			       paddle-x
			       (- paddle-y (/ g:+paddle-height+ 2))
			       g:+paddle-width+
			       g:+paddle-height+)
	      (point-in-rect-p (+ ball-x g:+ball-radius+)
			       (+ ball-y g:+ball-radius+)
			       paddle-x
			       (- paddle-y (/ g:+paddle-height+ 2))
			       g:+paddle-width+
			       g:+paddle-height+))
      (- ball-y paddle-y))))

(defun handle-ball (state timelapse)
  (let* ((ball (g:state-ball state))
	 (ball-x (g:ball-x ball))
	 (ball-y (g:ball-y ball))
	 (ball-vx (g:ball-vx ball))
	 (ball-vy (g:ball-vy ball))
	 (left-paddle (g:state-left-paddle state))
	 (right-paddle (g:state-right-paddle state))
	 (ball-left-paddle-collision
	   (get-ball-paddle-collision ball left-paddle 0.0))
	 (ball-right-paddle-collision
	   (get-ball-paddle-collision ball right-paddle (- 1.0 g:+paddle-width+))))
    (cond
      (ball-left-paddle-collision
       (g:launch-ball ball
		      (incf (g:state-bounces state))
		      (* (/ ball-left-paddle-collision
			    (+ (/ g:+paddle-height+ 2) g:+ball-radius+))
			 g:+max-launch-angle+)
		      (+ g:+paddle-width+ g:+ball-radius+)
		      ball-y))
      (ball-right-paddle-collision
       (g:launch-ball ball
		      (incf (g:state-bounces state))
		      (- pi
			 (* (/ ball-right-paddle-collision
			       (+ (/ g:+paddle-height+ 2) g:+ball-radius+))
			    (/ g:+max-launch-angle+ 2)))
		      (- 1.0 g:+paddle-width+ g:+ball-radius+)
		      ball-y))
      ((< (+ ball-x g:+ball-radius+) 0)
       (incf (g:state-right-score state))
       (g:random-launch-ball state))
      ((> (- ball-x g:+ball-radius+) 1)
       (incf (g:state-left-score state))
       (g:random-launch-ball state))
      ((< (- ball-y g:+ball-radius+) 0)
       (setf (g:ball-y ball) g:+ball-radius+)
       (setf (g:ball-vy ball) (* -1 ball-vy)))
      ((> (+ ball-y g:+ball-radius+) 1)
       (setf (g:ball-y ball) (- 1 g:+ball-radius+))
       (setf (g:ball-vy ball) (* -1 ball-vy)))
      (t
       (incf (g:ball-x ball) (* ball-vx timelapse))
       (incf (g:ball-y ball) (* ball-vy timelapse))))))

(defun computer-paddle-target (game)
  (let* ((game-state (g:game-state game))
	 (ball (g:state-ball game-state)))
    (if (plusp (g:ball-vx ball))
	(or (local-game-1p-computer-paddle-target game)
	    (setf (local-game-1p-computer-paddle-target game)
		(random g:+paddle-height+)))
	(setf (local-game-1p-computer-paddle-target game) nil))))

(defmethod g:on-init ((game local-game-1p))
  (g:random-launch-ball (g:game-state game)))

(defmethod g:on-update ((game local-game-1p) timelapse)
  (let ((game-state (g:game-state game)))
    (unless (g:state-paused game-state)
      (handle-left-player game-state timelapse
			  (list r:+key-w+ r:+key-up+)
			  (list r:+key-s+ r:+key-down+))
      (handle-right-computer game-state timelapse (computer-paddle-target game))
      (handle-ball game-state timelapse))))

(defmethod g:on-init ((game local-game-2p))
  (g:random-launch-ball (g:game-state game)))

(defmethod g:on-update ((game local-game-2p) timelapse)
  (let ((game-state (g:game-state game)))
    (unless (g:state-paused game-state)
      (handle-left-player game-state timelapse (list r:+key-w+) (list r:+key-s+))
      (handle-right-player game-state timelapse (list r:+key-up+) (list r:+key-down+))
      (handle-ball game-state timelapse))))

(defmethod g:on-quit ((game local-game-1p)))
(defmethod g:on-quit ((game local-game-2p)))
