(in-package :pong.client)

(defclass main-menu (scene)
  ((title           :initarg :title           :initform nil :reader main-menu-title)
   (items-group     :initarg :items-group     :initform nil :reader main-menu-items-group)
   (local-title     :initarg :local-title     :initform nil :reader main-menu-local-title)
   (local-1-player  :initarg :local-1-player  :initform nil :reader main-menu-local-1-player)
   (local-2-players :initarg :local-2-players :initform nil :reader main-menu-local-2-players)
   (online-title    :initarg :online-title    :initform nil :reader main-menu-online-title)
   (online-create   :initarg :online-create   :initform nil :reader main-menu-online-create)
   (online-join     :initarg :online-join     :initform nil :reader main-menu-online-join)))

(defmacro make-main-menu (&rest args)
  `(make-instance 'main-menu ,@args))

(defclass menu-item ()
  ((text   :initarg :text :initform nil :reader menu-item-text)
   (action :initarg :action :initform nil :reader menu-item-action)))

(defmacro make-menu-item ((&rest text-args) &key action)
  `(make-instance 'menu-item :action ,action
			     :text (gui:make-text ,@text-args)))

(defun open-main-menu ()
  (let* ((title (gui:make-text          :text "Pariatech's Pong Game"
			                :font *menu-font*
			                :font-size *menu-title-font-size*
			                :color *menu-text-color*
					:y *title-top-padding*
			                :h-align :center))
	 (local-title (gui:make-text    :text "Local"
				        :font *menu-font*
				        :font-size *menu-group-title-font-size*
				        :color *menu-text-color*
				        :h-align :center))
	 (1-player (make-menu-item      (:y (+ (gui:y local-title)
					       (gui:h local-title)
					       *menu-group-padding*)
					 :text *menu-local-1-player*
				         :font *menu-font*
				         :font-size *menu-font-size*
				         :color *menu-text-color*
				         :h-align :center)
					:action #'start-1-player-game))
	 (2-players (make-menu-item     (:y (+ (gui:y (menu-item-text 1-player))
					       (gui:h (menu-item-text 1-player))
					       *menu-padding*)
					 :text *menu-local-2-players*
					 :font *menu-font*
					 :font-size *menu-font-size*
					 :color *menu-text-color*
					 :h-align :center)
					:action #'start-2-players-game))
	 (online-title (gui:make-text   :y (+ (gui:y (menu-item-text 2-players))
					      (gui:h (menu-item-text 2-players))
					      *menu-group-padding*)
					:text "Online"
				        :font *menu-font*
				        :font-size *menu-group-title-font-size*
				        :color *menu-text-color*
				        :h-align :center))
	 (online-create (make-menu-item (:y (+ (gui:y online-title)
					       (gui:h online-title)
					       *menu-group-padding*)
					 :text *menu-online-create*
					 :font *menu-font*
					 :font-size *menu-font-size*
					 :color *menu-text-color*
					 :h-align :center)
					:action #'open-create-online-game-scene))
	 (online-join (make-menu-item   (:y (+ (gui:y (menu-item-text online-create))
					       (gui:h (menu-item-text online-create))
					       *menu-padding*)
					 :text *menu-online-join*
				         :font *menu-font*
				         :font-size *menu-font-size*
				         :color *menu-text-color*
				         :h-align :center)
					:action #'open-join-online-game-scene))
	 (items-group (gui:make-element :h (+ (gui:y (menu-item-text online-join))
					      (gui:h (menu-item-text online-join)))
					:h-align :center
					:v-align :middle))
	 (root-element (gui:make-rectangle :color r:+darkgray+
					   :h-align :center
					   :v-align :middle)))
    (gui:add-children items-group
		      local-title
		      (menu-item-text 1-player)
		      (menu-item-text 2-players)
		      online-title
		      (menu-item-text online-create)
		      (menu-item-text online-join))
    (gui:add-children root-element title items-group)
    (set-scene (make-main-menu :title title
			       :items-group items-group
			       :local-title local-title
			       :local-1-player 1-player
			       :local-2-players 2-players
			       :online-title online-title
			       :online-create online-create
			       :online-join online-join
			       :items-group items-group
			       :root-element root-element))))

(defun position-title (title)
  (setf (gui:x title) (floor (gui:w (gui:parent title)) 2)))

(defun position-items-group (items-group)
  (setf (gui:x items-group) (floor (gui:w (gui:parent items-group)) 2))
  (setf (gui:y items-group) (floor (gui:h (gui:parent items-group)) 2))
  (flet ((find-widest (children)
	   (let ((widest (car children)))
	     (dolist (child (cdr children) widest)
	       (when (> (gui:w child) (gui:w widest))
		 (setf widest child)))
	     widest)))
    (setf (gui:w items-group) (gui:w (find-widest (gui:children items-group))))))

(defun position-local-title (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun position-local-1-player (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun position-local-2-players (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun position-online-title (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun position-online-create (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun position-online-join (el)
  (setf (gui:x el) (floor (gui:w (gui:parent el)) 2)))

(defun update-text-if-hovered (el txt)
  (setf (gui:text el)
	(if (gui:hoveredp el)
	    (concatenate 'string "> " txt " <")
	    txt)))

(defun act-on-click (item)
  (when (gui:clickedp (menu-item-text item) r:+mouse-button-left+)
    (funcall (menu-item-action item))))

(defmethod on-update ((scene main-menu) timelapse)
  (with-slots (local-1-player local-2-players online-create online-join) scene
    (update-text-if-hovered (menu-item-text local-1-player) *menu-local-1-player*)
    (update-text-if-hovered (menu-item-text local-2-players) *menu-local-2-players*)
    (update-text-if-hovered (menu-item-text online-create) *menu-online-create*)
    (update-text-if-hovered (menu-item-text online-join) *menu-online-join*)
    (act-on-click local-1-player)
    (act-on-click local-2-players)
    (act-on-click online-create)
    (act-on-click online-join)
    (setf (scene-should-close scene) (r:is-key-pressed r:+key-escape+))))

(defmethod on-draw ((scene main-menu))
  (with-slots (root-element title items-group local-title local-1-player local-2-players online-title online-create online-join) scene
    (position-root root-element)
    (position-title title)
    (position-items-group items-group)
    (position-local-title local-title)
    (position-local-1-player (menu-item-text local-1-player))
    (position-local-2-players (menu-item-text local-2-players))
    (position-online-title online-title)
    (position-online-create (menu-item-text online-create))
    (position-online-join (menu-item-text online-join))))
